@@ -35,69 +35,70 @@ let sort_lineinfo line1 line2 =
  * to form a list of strings, each of which has length less than
  * 'len'. *)
 let word_wrap (s : string) (len : int) =
-   let ws = Str.regexp "[\t ]+" in
-   let split_words = Str.split ws s in
    let rec process_words words lines =
-      match words with
-      |[] ->
-         List.rev lines
-      |word :: remaining_words ->
-         let word_len = utf8_len word in
-         begin match lines with
-         |[] ->
-            process_words words [""]
-         |line :: remaining_lines ->
-            let line_len = utf8_len line in
-            if word_len + line_len + 1 <= len then
-               if line_len = 0 then
-                  process_words remaining_words (word :: remaining_lines)
-               else
-                  process_words remaining_words ((line ^ " " ^ word) :: remaining_lines)
-            else if word_len <= len then
-               process_words remaining_words (word :: line :: remaining_lines)
-            else
-               (* No choice but to break the word apart *)
-               let front = utf8_string_before word len
-               and back  = utf8_string_after  word len in
-               process_words (back :: remaining_words)
-                  (front :: line :: remaining_lines)
-         end
+      match words, lines with
+      | [], lines -> lines
+      | words, [] -> process_words words [""]
+      | word :: remaining_words, "" :: remaining_lines
+         when utf8_len word < len ->
+            process_words remaining_words (word :: remaining_lines)
+      | word :: remaining_words, line :: remaining_lines ->
+         let word_len, line_len = utf8_len word, utf8_len line in
+         if word_len + line_len < len then
+            process_words remaining_words
+               (String.concat " " [line; word] :: remaining_lines)
+         else if word_len <= len then
+            process_words remaining_words
+               (word :: line :: remaining_lines)
+         else
+            (* No choice but to break the word apart *)
+            let front = utf8_string_before word len
+            and back  = utf8_string_after  word len in
+            process_words (back :: remaining_words)
+               (front :: line :: remaining_lines)
    in
-   process_words split_words []
+   if len < 1 then [""] else
+   let ws = Str.regexp "[\t ]+" in
+   String.split_on_char '\n' s
+   |> ListLabels.fold_left ~init:[] ~f:(fun all_lines one_line ->
+         process_words (Str.split ws one_line) [] :: all_lines)
+   |> List.flatten
+   |> List.rev
 
 
 (* Generate a 12-hour clock representation of a time record *)
-let twelve_hour_string tm =
-   if tm.Unix.tm_hour >= 12 then
-      let hour = tm.Unix.tm_hour - 12 in
+let twelve_hour_string { Unix.tm_hour; tm_min; _ } =
+   if tm_hour >= 12 then
+      let hour = tm_hour - 12 in
       if hour = 0 then
-         Printf.sprintf "12:%.2dpm" tm.Unix.tm_min
+         Printf.sprintf "12:%.2dpm" tm_min
       else
-         Printf.sprintf "%d:%.2dpm" hour tm.Unix.tm_min
+         Printf.sprintf "%d:%.2dpm" hour tm_min
    else
-      if tm.Unix.tm_hour = 0 then
-         Printf.sprintf "12:%.2dam" tm.Unix.tm_min
+      if tm_hour = 0 then
+         Printf.sprintf "12:%.2dam" tm_min
       else
-         Printf.sprintf "%d:%.2dam" tm.Unix.tm_hour tm.Unix.tm_min
+         Printf.sprintf "%d:%.2dam" tm_hour tm_min
 
 
 (* Generate a 12-hour clock representation of a time record, with whitespace padding *)
-let twelve_hour_string_pad tm =
-   if tm.Unix.tm_hour >= 12 then
-      let hour = tm.Unix.tm_hour - 12 in
+let twelve_hour_string_pad { Unix.tm_hour; tm_min; _ } =
+   if tm_hour >= 12 then
+      let hour = tm_hour - 12 in
       if hour = 0 then
-         Printf.sprintf "12:%.2dpm" tm.Unix.tm_min
+         Printf.sprintf "12:%.2dpm" tm_min
       else
-         Printf.sprintf "%2d:%.2dpm" hour tm.Unix.tm_min
+         Printf.sprintf "%2d:%.2dpm" hour tm_min
    else
-      if tm.Unix.tm_hour = 0 then
-         Printf.sprintf "12:%.2dam" tm.Unix.tm_min
+      if tm_hour = 0 then
+         Printf.sprintf "12:%.2dam" tm_min
       else
-         Printf.sprintf "%2d:%.2dam" tm.Unix.tm_hour tm.Unix.tm_min
+         Printf.sprintf "%2d:%.2dam" tm_hour tm_min
 
 
 (* Generate a 24-hour clock representation of a time record *)
-let twentyfour_hour_string tm = Printf.sprintf "%.2d:%.2d" tm.Unix.tm_hour tm.Unix.tm_min
+let twentyfour_hour_string { Unix.tm_hour; tm_min; _ } =
+   Printf.sprintf "%.2d:%.2d" tm_hour tm_min
 
 
 (* Draw a string in a specified window and location, using exactly 'len'
@@ -115,7 +116,7 @@ let trunc_mvwaddstr win line col len s =
       else
          ""
    in
-   assert (mvwaddstr win line col fixed_s)
+   ignore (mvwaddstr win line col fixed_s)
 
 
 (* Draw the one-line help window at the top of the screen *)
@@ -228,30 +229,68 @@ let draw_date_strip (iface : interface_state_t) =
          (* if there are no date changes (e.g. for small window) then just grab the proper
           * date from the top_timestamp *)
          let tm = Unix.localtime iface.top_timestamp in
+         let len = if iface.scr.tw_lines > 6 then iface.scr.tw_lines - 6 else iface.scr.tw_lines in
          (Printf.sprintf " %s %.2d" (string_of_tm_mon tm.Unix.tm_mon)
-             tm.Unix.tm_mday) ^ (String.make (iface.scr.tw_lines - 6) ' ')
+             tm.Unix.tm_mday) ^ (String.make len ' ')
    in
    (* draw the date string vertically, one character at a time *)
    for i = 0 to pred iface.scr.tw_lines do
       if date_chars.[i] = '-' then begin
          wattron iface.scr.timed_win A.underline;
          Rcfile.color_on iface.scr.timed_win Rcfile.Timed_date;
-         assert (mvwaddch iface.scr.timed_win i 0 acs.Acs.hline);
+         ignore (mvwaddch iface.scr.timed_win i 0 (int_of_char ' '));
          Rcfile.color_off iface.scr.timed_win Rcfile.Timed_date;
          Rcfile.color_on iface.scr.timed_win Rcfile.Left_divider;
-         assert (mvwaddch iface.scr.timed_win i 1 acs.Acs.rtee);
+         ignore (mvwaddch iface.scr.timed_win i 1 acs.Acs.vline);
          Rcfile.color_off iface.scr.timed_win Rcfile.Left_divider;
          wattroff iface.scr.timed_win A.underline;
       end else begin
          Rcfile.color_on iface.scr.timed_win Rcfile.Timed_date;
-         assert (mvwaddch iface.scr.timed_win i 0 (int_of_char date_chars.[i]));
+         ignore (mvwaddch iface.scr.timed_win i 0 (int_of_char date_chars.[i]));
          Rcfile.color_off iface.scr.timed_win Rcfile.Timed_date;
       end
    done;
-   wattroff iface.scr.timed_win (A.bold lor A.underline);
+   wattroff iface.scr.timed_win A.bold;
    assert (wnoutrefresh iface.scr.timed_win)
 
 
+let rem_color_table = Hashtbl.create @@ color_pairs ()
+let color_counter =
+   let count = ref (Hashtbl.length Rcfile.color_table + 5) in
+   fun () ->
+      if !count < color_pairs () then begin
+         incr count;
+         !count
+      end else
+         0
+let color_key (r, g, b) = string_of_int r ^ string_of_int g ^ string_of_int b
+let color_index (r, g, b as color) =
+   let index = color_counter () in
+      if index > 0 then begin
+         (* scale from Remind colors (0-255) to ncurses colors (0-1000) *)
+         let scaled_r = int_of_float (float_of_int r *. 3.922)
+         and scaled_g = int_of_float (float_of_int g *. 3.922)
+         and scaled_b = int_of_float (float_of_int b *. 3.922) in
+         assert (init_color index scaled_r scaled_g scaled_b);
+         assert (init_pair index index 0);
+         Hashtbl.add rem_color_table (color_key color) index;
+         index
+      end else
+         0
+let hash_color_index color =
+   try Hashtbl.find rem_color_table (color_key color)
+   with Not_found -> color_index color
+let rgb_color win switch rgb_opt =
+   if Rcfile.reminder_colors `Is_enabled then begin
+   rgb_opt
+   |> Option.iter (fun rgb ->
+         let color_pair = A.color_pair @@ hash_color_index rgb in
+         match switch with
+         | `On  -> wattron  win color_pair
+         | `Off -> wattroff win color_pair
+      )
+   end
+
 (* Draw a portion of the timed schedule.  The algorithm iterates across all
  * reminders in a three month period, and draws them in one by one.  An array
  * is used to keep track of which lines have not yet been drawn, so that these
@@ -298,7 +337,8 @@ let draw_timed_window iface reminders top lines =
    Rcfile.color_on iface.scr.timed_win Rcfile.Timed_default;
    (* draw in the blank timeslots *)
    for i = top to pred (top + lines) do
-      iface.timed_lineinfo.(i) <- [];
+      if i >= 0 && i < Array.length iface.timed_lineinfo then
+         iface.timed_lineinfo.(i) <- [];
       if i = iface.left_selection && iface.selected_side = Left then
          wattron iface.scr.timed_win A.reverse
       else
@@ -318,12 +358,13 @@ let draw_timed_window iface reminders top lines =
          wattron iface.scr.timed_win A.underline
       else
          wattroff iface.scr.timed_win A.underline;
-      assert (mvwaddstr iface.scr.timed_win i 2 ts_str);
+      ignore (mvwaddstr iface.scr.timed_win i 2 ts_str);
       Rcfile.color_off iface.scr.timed_win Rcfile.Timed_current;
       wattroff iface.scr.timed_win A.bold;
       Rcfile.color_on iface.scr.timed_win Rcfile.Timed_default;
-      let s = Str.string_before blank (iface.scr.tw_cols - 7) in
-      assert (mvwaddstr iface.scr.timed_win i 7 s)
+      let len = if iface.scr.tw_cols > 7 then iface.scr.tw_cols - 7 else iface.scr.tw_cols in
+      let s = Str.string_before blank len in
+      ignore (mvwaddstr iface.scr.timed_win i 7 s)
    done;
    Rcfile.color_off iface.scr.timed_win Rcfile.Timed_default;
    wattroff iface.scr.timed_win (A.reverse lor A.underline);
@@ -345,18 +386,23 @@ let draw_timed_window iface reminders top lines =
             else
                (desc_string_of_tm1 (Unix.localtime rem.tr_start)) ^ " "
          in
-         (* draw the top line of a reminder *)
          let clock_pad = if !Rcfile.schedule_12_hour then 10 else 8 in
-         if rem_top_line >= top && rem_top_line < top + lines then begin
+         let draw_entry curr_line msg =
             let time_str = get_time_str () in
-            let ts = timestamp_of_line iface rem_top_line in
+            let curr_body = msg
+               |> Option.fold ~none:" " ~some:(fun m -> "  " ^ m) in
+            let ts = timestamp_of_line iface curr_line in
             let tm = Unix.localtime ts in
-            if rem_top_line = iface.left_selection && iface.selected_side = Left then
+            let last_entry_of_day =
+               tm.Unix.tm_hour = before_midnight.Unix.tm_hour &&
+               tm.Unix.tm_min  = before_midnight.Unix.tm_min
+            in
+            if curr_line = iface.left_selection &&
+               iface.selected_side = Left then
                wattron iface.scr.timed_win A.reverse
             else
                wattroff iface.scr.timed_win A.reverse;
-            if tm.Unix.tm_hour = before_midnight.Unix.tm_hour &&
-               tm.Unix.tm_min  = before_midnight.Unix.tm_min then
+            if last_entry_of_day then
                wattron iface.scr.timed_win A.underline
             else
                wattroff iface.scr.timed_win A.underline;
@@ -364,53 +410,37 @@ let draw_timed_window iface reminders top lines =
                tl_filename = rem.tr_filename;
                tl_linenum  = rem.tr_linenum;
                tl_timestr  = time_str;
+               tl_cal      = rem.tr_cal;
                tl_msg      = rem.tr_msg;
                tl_start    = rem.tr_start
             } in
-            iface.timed_lineinfo.(rem_top_line) <-
-               (curr_lineinfo :: iface.timed_lineinfo.(rem_top_line));
-            trunc_mvwaddstr iface.scr.timed_win rem_top_line (clock_pad + (9 * indent))
-               (iface.scr.tw_cols - clock_pad - (9 * indent)) ("  " ^ rem.tr_msg);
-            assert (mvwaddch iface.scr.timed_win rem_top_line
-               (clock_pad + (9 * indent)) acs.Acs.vline)
-         end else
-            ();
+            iface.timed_lineinfo.(curr_line) <-
+               (curr_lineinfo :: iface.timed_lineinfo.(curr_line));
+            if Option.is_some msg then
+               rgb_color iface.scr.timed_win `On rem.tr_rgb;
+            trunc_mvwaddstr iface.scr.timed_win curr_line
+               (clock_pad + (9 * indent))
+               (iface.scr.tw_cols - clock_pad - (9 * indent))
+               curr_body;
+            if Option.is_some msg then
+               rgb_color iface.scr.timed_win `Off rem.tr_rgb;
+            ignore (mvwaddch iface.scr.timed_win curr_line
+               (clock_pad + (9 * indent)) acs.Acs.vline);
+            if last_entry_of_day then
+               wattroff iface.scr.timed_win A.underline
+         in
+         (* draw the top line of a reminder *)
+         if rem_top_line >= top && rem_top_line < top + lines then
+            draw_entry rem_top_line (Some rem.tr_cal);
          (* draw any remaining lines of this reminder, as determined by the duration *)
-         let count = ref 1 in
+         let curr_line = ref (rem_top_line + 1) in
          while
-            ((timestamp_of_line iface (rem_top_line + !count)) < rem.tr_end) &&
-            (rem_top_line + !count < top + lines)
+            (timestamp_of_line iface !curr_line < rem.tr_end) &&
+            (!curr_line < top + lines)
          do
-            if rem_top_line + !count >= top then begin
-               let time_str = get_time_str () in
-               let ts = timestamp_of_line iface (rem_top_line + !count) in
-               let tm = Unix.localtime ts in
-               if rem_top_line + !count = iface.left_selection &&
-                  iface.selected_side = Left then
-                  wattron iface.scr.timed_win A.reverse
-               else
-                  wattroff iface.scr.timed_win A.reverse;
-               if tm.Unix.tm_hour = before_midnight.Unix.tm_hour &&
-                  tm.Unix.tm_min  = before_midnight.Unix.tm_min then
-                  wattron iface.scr.timed_win A.underline
-               else
-                  wattroff iface.scr.timed_win A.underline;
-               let curr_lineinfo = {
-                  tl_filename = rem.tr_filename;
-                  tl_linenum  = rem.tr_linenum;
-                  tl_timestr  = time_str;
-                  tl_msg      = rem.tr_msg;
-                  tl_start    = rem.tr_start
-               } in
-               iface.timed_lineinfo.(rem_top_line + !count) <-
-                  (curr_lineinfo :: iface.timed_lineinfo.(rem_top_line + !count));
-               trunc_mvwaddstr iface.scr.timed_win (rem_top_line + !count)
-                  (clock_pad + (9 * indent)) (iface.scr.tw_cols - clock_pad - (9 * indent)) " ";
-               assert (mvwaddch iface.scr.timed_win (rem_top_line + !count)
-                  (clock_pad + (9 * indent)) acs.Acs.vline)
-            end else
-               ();
-            count := succ !count
+            if !curr_line >= top then
+               draw_entry !curr_line None;
+            incr curr_line
          done;
          (* The reminders list is sorted chronologically, so once we hit a reminder
           * that falls after the end of the calendar window we can stop. *)
@@ -447,10 +477,11 @@ let draw_timed_try_window iface reminders top lines =
    let (ts, _)  = Unix.mktime timeslot in
    if iface.last_timed_refresh < ts then
       draw_timed iface reminders
-   else begin
+   else if top >= 0 && top < Array.length iface.timed_lineinfo then begin
       draw_timed_window iface reminders top lines;
       iface
-   end
+   end else
+      iface
 
 
 (* render a calendar for the given reminders record *)
@@ -466,15 +497,15 @@ let draw_calendar (iface : interface_state_t)
    Rcfile.color_off iface.scr.calendar_win Rcfile.Right_divider;
    let hspacer = (iface.scr.cw_cols - 20) / 2 in
    let vspacer = (iface.scr.cw_lines - 8) / 2 in
-   assert (wmove iface.scr.calendar_win vspacer hspacer);
+   ignore (wmove iface.scr.calendar_win vspacer hspacer);
    wclrtoeol iface.scr.calendar_win;
    Rcfile.color_on iface.scr.calendar_win Rcfile.Calendar_labels;
    wattron iface.scr.calendar_win A.bold;
-   assert (waddstr iface.scr.calendar_win cal.Cal.title);
+   ignore (waddstr iface.scr.calendar_win cal.Cal.title);
    wattroff iface.scr.calendar_win A.bold;
-   assert (wmove iface.scr.calendar_win (vspacer + 1) hspacer);
+   ignore (wmove iface.scr.calendar_win (vspacer + 1) hspacer);
    wclrtoeol iface.scr.calendar_win;
-   assert (waddstr iface.scr.calendar_win cal.Cal.weekdays);
+   ignore (waddstr iface.scr.calendar_win cal.Cal.weekdays);
    Rcfile.color_off iface.scr.calendar_win Rcfile.Calendar_labels;
    (* draw the day numbers *)
    let ws = Str.regexp " " in
@@ -580,7 +611,7 @@ let draw_untimed (iface : interface_state_t) reminders =
    let acs = get_acs_codes () in
    Rcfile.color_on iface.scr.untimed_win Rcfile.Right_divider;
    wattron iface.scr.untimed_win A.bold;
-   assert (mvwaddch iface.scr.untimed_win 0 0 acs.Acs.ltee);
+   ignore (mvwaddch iface.scr.untimed_win 0 0 acs.Acs.ltee);
    mvwhline iface.scr.untimed_win 0 1 acs.Acs.hline (pred iface.scr.uw_cols);
    mvwvline iface.scr.untimed_win 1 0 acs.Acs.vline (pred iface.scr.uw_lines);
    Rcfile.color_off iface.scr.untimed_win Rcfile.Right_divider;
@@ -617,8 +648,10 @@ let draw_untimed (iface : interface_state_t) reminders =
                   wattron iface.scr.untimed_win A.reverse
                else
                   wattroff iface.scr.untimed_win A.reverse;
+               rgb_color iface.scr.untimed_win `On rem.ur_rgb;
                trunc_mvwaddstr iface.scr.untimed_win line 2 (iface.scr.uw_cols - 3)
-                  ("* " ^ rem.ur_msg);
+                  ("* " ^ rem.ur_cal);
+               rgb_color iface.scr.untimed_win `Off rem.ur_rgb;
                let curr_lineinfo = {
                   ul_filename = rem.ur_filename;
                   ul_linenum  = rem.ur_linenum;
@@ -637,18 +670,20 @@ let draw_untimed (iface : interface_state_t) reminders =
    (* if there's not enough window space to display all untimed reminders, display
     * arrows to indicate scrolling ability *)
    if iface.top_untimed > 0 then begin
-      assert (mvwaddch iface.scr.untimed_win 1 (pred iface.scr.uw_cols) (int_of_char '^'));
-      assert (mvwaddch iface.scr.untimed_win 2 (pred iface.scr.uw_cols) (int_of_char '^'))
+      ignore (mvwaddch iface.scr.untimed_win 1 (pred iface.scr.uw_cols) (int_of_char '^'));
+      ignore (mvwaddch iface.scr.untimed_win 2 (pred iface.scr.uw_cols) (int_of_char '^'))
    end else
       ();
-   if List.length today_reminders > pred iface.scr.uw_lines + iface.top_untimed then begin
-      assert (mvwaddch iface.scr.untimed_win (pred iface.scr.uw_lines)
+   if iface.scr.uw_cols > 2 && iface.scr.uw_lines > 1 &&
+      List.length today_reminders > pred iface.scr.uw_lines + iface.top_untimed then begin
+      ignore (mvwaddch iface.scr.untimed_win (pred iface.scr.uw_lines)
              (pred iface.scr.uw_cols) (int_of_char 'v'));
-      assert (mvwaddch iface.scr.untimed_win (iface.scr.uw_lines - 2)
-             (pred iface.scr.uw_cols) (int_of_char 'v'))
+      if iface.scr.uw_lines > 2 then
+         ignore (mvwaddch iface.scr.untimed_win (iface.scr.uw_lines - 2)
+                (pred iface.scr.uw_cols) (int_of_char 'v'))
    end else
       ();
-   assert (wnoutrefresh iface.scr.untimed_win);
+   ignore (wnoutrefresh iface.scr.untimed_win);
    {iface with untimed_lineinfo = lineinfo;
                len_untimed      = List.length today_reminders}
 
@@ -674,7 +709,7 @@ let draw_msg iface =
          day_s
    in
    for i = 0 to pred iface.scr.mw_lines do
-      assert (wmove iface.scr.msg_win i 0);
+      ignore (wmove iface.scr.msg_win i 0);
       wclrtoeol iface.scr.msg_win
    done;
    (* draw the date stamp *)
@@ -695,7 +730,7 @@ let draw_msg iface =
    in
    let s =
       Printf.sprintf "Wyrd v%s          Currently: %s, %s %.2d at %s"
-         Install.version (full_string_of_tm_wday curr_tm.Unix.tm_wday)
+         iface.version (full_string_of_tm_wday curr_tm.Unix.tm_wday)
          (full_string_of_tm_mon curr_tm.Unix.tm_mon)
          curr_tm.Unix.tm_mday curr_tm_str
    in
@@ -756,7 +791,7 @@ let draw_msg iface =
          if start > 0 then
             draw_desc_lines tail (pred start) line_num
          else if line_num < pred iface.scr.mw_lines then begin
-            assert (mvwaddstr iface.scr.msg_win line_num 5 line);
+            ignore (mvwaddstr iface.scr.msg_win line_num 5 line);
             draw_desc_lines tail start (succ line_num)
          end else
             ()
@@ -772,9 +807,9 @@ let draw_msg iface =
    end else
       ();
    if adjusted_top < List.length desc_lines - iface.scr.mw_lines + 2 then begin
-      assert (mvwaddch iface.scr.msg_win (iface.scr.mw_lines - 2)
+      ignore (mvwaddch iface.scr.msg_win (iface.scr.mw_lines - 2)
          (iface.scr.mw_cols - 1) (int_of_char 'v'));
-      assert (mvwaddch iface.scr.msg_win (iface.scr.mw_lines - 3)
+      ignore (mvwaddch iface.scr.msg_win (iface.scr.mw_lines - 3)
          (iface.scr.mw_cols - 1) (int_of_char 'v'))
    end else
       ();
