@@ -22,6 +22,6 @@ type t = LC_ALL | LC_COLLATE | LC_CTYPE | LC_MONETARY |
          LC_NUMERIC | LC_TIME | LC_MESSAGES |
          LC_UNDEFINED of int
 
-external setlocale_int : int -> string -> string = "ml_setlocale"
+external setlocale_int : int -> string -> string option = "ml_setlocale"
 
-val setlocale : t -> string -> string
+val setlocale : t -> string -> string option
