@@ -31,6 +31,11 @@ value ml_setlocale(value category, value setting)
    CAMLparam2(category, setting);
    CAMLlocal1(result);
 
-   result = caml_copy_string(setlocale(Int_val(category), String_val(setting)));
+   char* loc = setlocale(Int_val(category), String_val(setting));
+   if (!loc) {
+      result = Val_int(0); // None
+   } else {
+      result = caml_alloc_boxed(caml_copy_string(loc)); // Some string
+   }
    CAMLreturn(result);
 }
