@@ -38,10 +38,13 @@ module IPSet = Set.Make (IntPair)
 type timed_rem_t = {
    tr_start      : float;
    tr_end        : float;
+   tr_duration   : float;
+   tr_cal        : string;
    tr_msg        : string;
    tr_filename   : string;
    tr_linenum    : string;
-   tr_has_weight : bool
+   tr_has_weight : bool;
+   tr_rgb        : (int * int * int) option;
 }
 
 
@@ -49,10 +52,12 @@ type timed_rem_t = {
  * the output of 'remind -s'. *)
 type untimed_rem_t = {
    ur_start      : float;
+   ur_cal        : string;
    ur_msg        : string;
    ur_filename   : string;
    ur_linenum    : string;
-   ur_has_weight : bool
+   ur_has_weight : bool;
+   ur_rgb        : (int * int * int) option;
 }
 
 
@@ -104,13 +109,8 @@ let month_start_of_tm tm =
  * (number of reminders) * (average reminder duration).  Since indentation is
  * determined a month at a time, there may be some minor discrepancies at
  * month borders. *)
-let process_timed tm duration_s month_start_ts indentations partial_trem
-timed =
-   let (f_rem_ts, _) = Unix.mktime tm in
-   let duration =
-      if duration_s = "*" then 0.0
-      else float_of_string duration_s
-   in
+let process_timed month_start_ts indentations trem timed =
+   let f_rem_ts, duration = trem.tr_start, trem.tr_duration in
    (* compute the indentation level *)
    (* top_index and bottom_index provide the range of row indices into
     * array indentations that are covered by this reminder *)
@@ -164,13 +164,89 @@ timed =
          pred (Array.length indentations.(0))
    in
    let indent = find_indent 0 in
-   let trem = {partial_trem with
-      tr_start      = f_rem_ts;
-      tr_end        = f_rem_ts +. (duration *. 60.);
-   } in
    timed.(indent) <- trem :: timed.(indent)
 
 
+let parse_remind_json rem_line =
+   let nodisplay_regex = Str.regexp_case_fold ".*nodisplay" in
+   let noweight_regex = Str.regexp_case_fold ".*noweight" in
+   let regex_tags tag_list str =
+      List.fold_left (fun tags (regex, tag) ->
+         if Str.string_match regex str 0
+         then tag :: tags
+         else tags) [] tag_list
+   in
+   let open Yojson.Basic.Util in
+   let line_num_s = rem_line |> member "lineno" |> to_int |> string_of_int
+   and filename   = rem_line |> member "filename" |> to_string
+   and date_s     = rem_line |> member "date" |> to_string
+   and special    = rem_line |> member "passthru" |> to_string_option
+      |> Option.fold ~none:false ~some:(fun sp ->
+         match String.lowercase_ascii sp with
+         | "color" | "colour" -> false
+         | _ -> true)
+   and tags       = rem_line |> member "tags" |> to_string_option
+      |> Option.fold ~none:[] ~some:(regex_tags
+         [nodisplay_regex, `Nodisplay; noweight_regex, `Noweight])
+   and rgb        = match rem_line |> member "r", rem_line |> member "g", rem_line |> member "b" with
+      | `Int r, `Int g, `Int b -> Some (r, g, b)
+      | _ -> None
+   and plain_body = match rem_line |> member "plain_body" with
+      | `String s -> s
+      | _ -> rem_line |> member "body" |> to_string in
+   let calendar_body = rem_line |> member "calendar_body" |> to_string_option
+      |> Option.value ~default:plain_body in
+   let cal_msg_no_newline = String.map (function '\n' -> ' ' | c -> c) calendar_body in
+   (* further subdivide the date string *)
+   let date_arr = Array.of_list (Str.split (Str.regexp "[/-]") date_s) in
+   let year     = int_of_string date_arr.(0)
+   and month    = int_of_string date_arr.(1)
+   and day      = int_of_string date_arr.(2) in
+   let temp = {empty_tm with
+      Unix.tm_mday  = day;
+      Unix.tm_mon   = pred month;
+      Unix.tm_year  = year - 1900;
+   } in
+   (* check whether this reminder is special or tagged 'nodisplay' *)
+   if special || List.mem `Nodisplay tags then
+      (* skip this reminder due to special or 'nodisplay' tag *)
+      `Skip
+   else begin
+      let has_weight = not (List.mem `Noweight tags) in
+      if rem_line |> member "time" = `Null then
+         (* if minutes are not provided, this must be an untimed reminder *)
+         let (f_rem_ts, _) = Unix.mktime temp in
+         let urem = {
+            ur_start      = f_rem_ts;
+            ur_cal        = cal_msg_no_newline;
+            ur_msg        = plain_body;
+            ur_filename   = filename;
+            ur_linenum    = line_num_s;
+            ur_has_weight = has_weight;
+            ur_rgb        = rgb;
+         } in
+         `Untimed urem
+      else begin
+         (* if minutes are provided, this must be a timed reminder *)
+         let min = rem_line |> member "time" |> to_int in
+         let duration = rem_line |> member "duration" |> to_int_option
+            |> Option.fold ~none:0. ~some:float_of_int in
+         let temp_with_min = {temp with Unix.tm_min = min} in
+         let (f_rem_ts, _) = Unix.mktime temp_with_min in
+         let trem = {
+            tr_start      = f_rem_ts;
+            tr_end        = f_rem_ts +. (duration *. 60.);
+            tr_duration   = duration;
+            tr_cal        = cal_msg_no_newline;
+            tr_msg        = plain_body;
+            tr_filename   = filename;
+            tr_linenum    = line_num_s;
+            tr_has_weight = has_weight;
+            tr_rgb        = rgb;
+         } in
+         `Timed trem
+      end
+   end
 
 (* Obtain two lists of reminders for the month of the timestamp argument.
  * The first list is for timed reminders, the second is for untimed.
@@ -188,107 +264,61 @@ timed =
  * The optional argument 'suppress_advwarn', if true, will override
  * the "advance_warning" rcfile option. *)
 let month_reminders ?(suppress_advwarn=false) timestamp =
-   let comment_regex = Str.regexp "^#.*fileinfo \\([^ ]+\\) \\(.*\\)$" in
-   let rem_regex = Str.regexp "\\([^ ]+\\) [^ ]+ \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\) \\(.*\\)$" in
-   let nodisplay_regex = Str.regexp_case_fold ".*nodisplay" in
-   let noweight_regex = Str.regexp_case_fold ".*noweight" in
    let tm = Unix.localtime timestamp in
-   let rem_date_str = (string_of_tm_mon tm.Unix.tm_mon) ^ " " ^
-                      (string_of_int tm.Unix.tm_mday) ^ " " ^
-                      (string_of_int (tm.Unix.tm_year + 1900)) in
-   let remind_s_flag =
-      if suppress_advwarn then " -s"
-      else if !Rcfile.advance_warning then " -sa"
-      else " -s" in
-   let full_remind_command =
-      !Rcfile.remind_command ^ remind_s_flag ^ " -l -g -b2 " ^
-      !Rcfile.reminders_file ^ " " ^ rem_date_str
-   in
-   let (out_lines, err_lines) = Utility.read_all_shell_command_output full_remind_command in
-   (* check for Remind errors *)
-   let remind_err = if List.length err_lines > 0 then List.hd err_lines else "" in
+   let (month_start_ts, _) = month_start_of_tm tm in
    let num_indentations = 4 in
    let indentations = Array.make_matrix (24 * 31) num_indentations false in
-   let (month_start_ts, _) = month_start_of_tm tm in
    let timed = Array.make num_indentations [] in
    let rec build_lists lines untimed =
       begin match lines with
       | [] ->
-         for i = 0 to pred (Array.length timed) do
-            timed.(i) <- List.rev timed.(i)
-         done;
-         (remind_err, timed, List.rev untimed)
-      | comment_line :: rem_line :: lines_tail ->
-         begin try
-            if Str.string_match comment_regex comment_line 0 then begin
-               let line_num_s = Str.matched_group 1 comment_line
-               and filename   = Str.matched_group 2 comment_line in
-               if Str.string_match rem_regex rem_line 0 then begin
-                  let date_s     = Str.matched_group 1 rem_line
-                  and tag        = Str.matched_group 2 rem_line
-                  and duration_s = Str.matched_group 3 rem_line
-                  and min_s      = Str.matched_group 4 rem_line
-                  and msg        = Str.matched_group 5 rem_line in
-                  (* further subdivide the date string *)
-                  let date_arr = Array.of_list (Str.split (Str.regexp "[/-]") date_s) in
-                  let year     = int_of_string date_arr.(0)
-                  and month    = int_of_string date_arr.(1)
-                  and day      = int_of_string date_arr.(2) in
-                  let temp = {empty_tm with
-                     Unix.tm_mday  = day;
-                     Unix.tm_mon   = pred month;
-                     Unix.tm_year  = year - 1900;
-                  } in
-                  (* check whether this reminder is tagged 'nodisplay' *)
-                  if (Str.string_match nodisplay_regex tag 0) then
-                     (* skip this reminder due to a 'nodisplay' tag *)
-                     build_lists lines_tail untimed
-                  else begin
-                     let has_weight = not (Str.string_match noweight_regex tag 0) in
-                     if min_s = "*" then
-                        (* if minutes are not provided, this must be an untimed reminder *)
-                        let (f_rem_ts, _) = Unix.mktime temp in
-                        let urem = {
-                           ur_start      = f_rem_ts;
-                           ur_msg        = msg;
-                           ur_filename   = filename;
-                           ur_linenum    = line_num_s;
-                           ur_has_weight = has_weight
-                        } in
-                        build_lists lines_tail (urem :: untimed)
-                     else begin
-                        (* if minutes are provided, this must be a timed reminder *)
-                        let temp_with_min = {temp with Unix.tm_min = int_of_string min_s} in
-                        let partial_trem = {
-                           tr_start      = 0.0;  (* still needs to be filled in *)
-                           tr_end        = 0.0;  (* still needs to be filled in *)
-                           tr_msg        = msg;
-                           tr_filename   = filename;
-                           tr_linenum    = line_num_s;
-                           tr_has_weight = has_weight
-                        } in
-                        process_timed temp_with_min duration_s month_start_ts indentations
-                        partial_trem timed;
-                        build_lists lines_tail untimed
-                     end
-                  end
-               end else
-                  (* if there was no rem_regex match, continue with next line *)
-                  build_lists (rem_line :: lines_tail) untimed
-            end else
-               (* if there was no comment_regex match, continue with next line *)
-               build_lists (rem_line :: lines_tail) untimed
-         with _ ->
-            (* if there's an error in regexp matching or string coersion,
-             * just drop that reminder and go to the next line *)
-            build_lists (rem_line :: lines_tail) untimed
+         let () =
+            for i = 0 to pred (Array.length timed) do
+               timed.(i) <- List.rev timed.(i)
+            done
+         in
+         List.rev untimed
+      | rem_line :: lines_tail ->
+         begin match parse_remind_json rem_line with
+         | `Skip -> build_lists lines_tail untimed
+         | `Untimed urem -> build_lists lines_tail (urem :: untimed)
+         | `Timed trem ->
+            let () =
+               process_timed month_start_ts indentations trem timed
+            in
+            build_lists lines_tail untimed
          end
-      | comment_line :: [] ->
-         (* this line doesn't conform to the spec, so throw it out *)
-         build_lists [] untimed
       end
    in
-   build_lists out_lines []
+   let rem_date_str = String.concat " "
+                         [string_of_tm_mon tm.Unix.tm_mon;
+                          string_of_int tm.Unix.tm_mday;
+                          string_of_int (tm.Unix.tm_year + 1900)] in
+   let remind_s_flag =
+      if not suppress_advwarn && !Rcfile.advance_warning then "-pppqa"
+      else "-pppq" in
+   let fname = !Rcfile.reminders_file in
+   let full_remind_command =
+      String.concat " "
+         [!Rcfile.remind_command; remind_s_flag;
+          "-l -g -b2"; fname; rem_date_str]
+   in
+   let (out_lines, err_lines) =
+      Utility.read_all_shell_command_output full_remind_command in
+   (* check for Remind errors *)
+   let remind_err = List.(if length err_lines > 0 then hd err_lines else "") in
+   (* avoid unhelpful exception Json_error("Blank input data"),
+    * as `remind_err` should tell us why `out_lines` is blank *)
+   if List.length out_lines > 0 then
+      let json = Yojson.Basic.from_string ~fname (String.concat "" out_lines) in
+      let entries = Yojson.Basic.Util.(
+         json |> to_list |> List.hd |> member "entries" |> to_list) in
+      (* use let ... in to ensure mutable Array `timed`
+       * is updated by the call to `build_lists` *)
+      let untimed = build_lists entries [] in
+      (remind_err, timed, untimed)
+   else
+      (remind_err, timed, [])
 
 
 (* generate a count of how many reminders fall on any given day of
@@ -400,12 +430,12 @@ let safe_append a b = List.rev_append (List.rev a) b
 
 
 (* initialize a new three-month reminder record *)
-let create_three_month ?(suppress_advwarn=false) timestamp =
+let create_three_month ?suppress_advwarn timestamp =
    let month_start_tm = {
-      Unix.localtime timestamp with Unix.tm_sec  = 0;
-                                    Unix.tm_min  = 0;
-                                    Unix.tm_hour = 0;
-                                    Unix.tm_mday = 1
+      (Unix.localtime timestamp) with Unix.tm_sec  = 0;
+                                      Unix.tm_min  = 0;
+                                      Unix.tm_hour = 0;
+                                      Unix.tm_mday = 1
    } in
    let (curr_ts, _) = Unix.mktime month_start_tm in
    let temp_prev = {
@@ -416,9 +446,9 @@ let create_three_month ?(suppress_advwarn=false) timestamp =
    } in
    let (prev_ts, _) = Unix.mktime temp_prev
    and (next_ts, _) = Unix.mktime temp_next in
-   let (pre, pt, pu) = month_reminders ~suppress_advwarn:suppress_advwarn prev_ts in
-   let (cre, ct, cu) = month_reminders ~suppress_advwarn:suppress_advwarn curr_ts in
-   let (nre, nt, nu) = month_reminders ~suppress_advwarn:suppress_advwarn next_ts in
+   let (pre, pt, pu) = month_reminders ?suppress_advwarn prev_ts in
+   let (cre, ct, cu) = month_reminders ?suppress_advwarn curr_ts in
+   let (nre, nt, nu) = month_reminders ?suppress_advwarn next_ts in
    let at = Array.make (Array.length pt) [] in
    for i = 0 to pred (Array.length at) do
       at.(i) <- safe_append pt.(i) (safe_append ct.(i) nt.(i))
@@ -451,14 +481,18 @@ let create_three_month ?(suppress_advwarn=false) timestamp =
 (* Update a three-month reminders record for the next month *)
 let next_month reminders =
    let curr_tm = Unix.localtime reminders.curr_timestamp in
-   let temp1 = {
-      curr_tm with Unix.tm_mon = succ curr_tm.Unix.tm_mon
-   } in
-   let temp2 = {
-      curr_tm with Unix.tm_mon = curr_tm.Unix.tm_mon + 2
-   } in
-   let (new_curr_timestamp, temp1) = Unix.mktime temp1 in
-   let (next_timestamp, temp2)     = Unix.mktime temp2 in
+   let (new_curr_timestamp, temp1) =
+      let temp1 = {
+         curr_tm with Unix.tm_mon = succ curr_tm.Unix.tm_mon
+      } in
+      Unix.mktime temp1
+   in
+   let (next_timestamp, _temp2)     =
+      let temp2 = {
+         curr_tm with Unix.tm_mon = curr_tm.Unix.tm_mon + 2
+      } in
+      Unix.mktime temp2
+   in
    let (re, t, u) = month_reminders next_timestamp in
    let at = Array.make (Array.length t) [] in
    for i = 0 to pred (Array.length t) do
@@ -484,14 +518,18 @@ let next_month reminders =
 (* Update a three-month reminders record for the previous month *)
 let prev_month reminders =
    let curr_tm = Unix.localtime reminders.curr_timestamp in
-   let temp1 = {
-      curr_tm with Unix.tm_mon = pred curr_tm.Unix.tm_mon
-   } in
-   let temp2 = {
-      curr_tm with Unix.tm_mon = curr_tm.Unix.tm_mon - 2
-   } in
-   let (new_curr_timestamp, temp1) = Unix.mktime temp1 in
-   let (prev_timestamp, temp2)     = Unix.mktime temp2 in
+   let (new_curr_timestamp, temp1) =
+      let temp1 = {
+         curr_tm with Unix.tm_mon = pred curr_tm.Unix.tm_mon
+      } in
+      Unix.mktime temp1
+   in
+   let (prev_timestamp, _temp2)     =
+      let temp2 = {
+         curr_tm with Unix.tm_mon = curr_tm.Unix.tm_mon - 2
+      } in
+      Unix.mktime temp2
+   in
    let (re, t, u) = month_reminders prev_timestamp in
    let at = Array.make (Array.length t) [] in
    for i = 0 to pred (Array.length t) do
@@ -555,21 +593,25 @@ let update_reminders rem timestamp =
  * should not be able to match them. *)
 let find_next msg_regex timestamp =
    let rem_regex =
-      Str.regexp "^\\([^ ]+\\)[/-]\\([^ ]+\\)[/-]\\([^ ]+\\) [^ ]+ \\([^ ]+\\) [^ ]+ \\([^ ]+\\) \\([^ ]+.*\\)$"
+      Str.regexp {|^\([^ ]+\)[/-]\([^ ]+\)[/-]\([^ ]+\) [^ ]+ \([^ ]+\) [^ ]+ \([^ ]+\) \([^ ]+.*\)$|}
    in
    let nodisplay_regex = Str.regexp_case_fold ".*nodisplay" in
    let tm1 = Unix.localtime timestamp in
    let temp = {tm1 with Unix.tm_mday = succ tm1.Unix.tm_mday} in     (* add 24 hours *)
    let (_, tm2) = Unix.mktime temp in
-   let rem_date_str1 = (string_of_tm_mon tm1.Unix.tm_mon) ^ " " ^
-                       (string_of_int tm1.Unix.tm_mday) ^ " " ^
-                       (string_of_int (tm1.Unix.tm_year + 1900)) in
-   let rem_date_str2 = (string_of_tm_mon tm2.Unix.tm_mon) ^ " " ^
-                       (string_of_int tm2.Unix.tm_mday) ^ " " ^
-                       (string_of_int (tm2.Unix.tm_year + 1900)) in
+   let rem_date_str1 = String.concat " "
+                          [string_of_tm_mon tm1.Unix.tm_mon;
+                           string_of_int tm1.Unix.tm_mday;
+                           string_of_int (tm1.Unix.tm_year + 1900)] in
+   let rem_date_str2 = String.concat " "
+                          [string_of_tm_mon tm2.Unix.tm_mon;
+                           string_of_int tm2.Unix.tm_mday;
+                           string_of_int (tm2.Unix.tm_year + 1900)] in
    let remind_output_for_month date_str =
       let remind_month_command =
-         !Rcfile.remind_command ^ " -n -s -b1 " ^ !Rcfile.reminders_file ^ " " ^ date_str
+         String.concat " "
+            [!Rcfile.remind_command; "-n -s -b1";
+             !Rcfile.reminders_file; date_str]
       in
       let (out_lines, _) = Utility.read_all_shell_command_output remind_month_command in
       out_lines
@@ -619,7 +661,7 @@ let find_next msg_regex timestamp =
                (* if there was no regexp match, continue with next line *)
                check_messages lines_tail
          with
-         | Failure s ->
+         | Failure _s ->
            (* if there's an error in string coersion, just drop that reminder and go
             * to the next line *)
            check_messages lines_tail
