@@ -42,32 +42,31 @@ type event_t = Timed of (Unix.tm * Unix.tm) | Untimed of Unix.tm
 (* Note: careful when changing these regexps.  If the grouping tree changes,
  * then the fields may be misnumbered in code below. *)
 
-(* Note: ocaml _really_ needs a raw string syntax to make regexps look more sane... *)
 let weekdays =
-   "\\(\\(on[ \t]+\\)?\\(next[ \t]+\\)?" ^
-   "\\(\\(sunday\\|sun\\)\\|\\(monday\\|mon\\)\\|\\(tuesday\\|tue\\|tues\\)\\|" ^
-   "\\(wednesday\\|wed\\)\\|\\(thursday\\|thu\\|thur\\|thurs\\)\\|" ^
-   "\\(friday\\|fri\\)\\|\\(saturday\\|sat\\)\\)\\)"
+   {|\(\(on|} ^ "[ \t]+" ^ {|\)?\(next|} ^ "[ \t]+" ^ {|\)?|} ^
+   {|\(\(sunday\|sun\)\|\(monday\|mon\)\|\(tuesday\|tue\|tues\)\||} ^
+   {|\(wednesday\|wed\)\|\(thursday\|thu\|thur\|thurs\)\||} ^
+   {|\(friday\|fri\)\|\(saturday\|sat\)\)\)|}
 (* typical US-style dates, like 6/1 or 6/1/2006 *)
-let numeric_slash_date = "\\(\\([0-9]+\\)/\\([0-9]+\\)\\(/\\([0-9][0-9][0-9][0-9]\\)\\)?\\)"
+let numeric_slash_date = {|\(\([0-9]+\)/\([0-9]+\)\(/\([0-9][0-9][0-9][0-9]\)\)?\)|}
 (* ISO standard dates, like 2006-06-01 *)
-let iso_date = "\\(\\([0-9][0-9][0-9][0-9]\\)-\\([0-9]+\\)-\\([0-9]+\\)\\)"
-let numeric_date = "\\(\\(on[ \t]+\\)?\\(" ^ numeric_slash_date ^ "\\|" ^ iso_date ^ "\\)\\)"
+let iso_date = {|\(\([0-9][0-9][0-9][0-9]\)-\([0-9]+\)-\([0-9]+\)\)|}
+let numeric_date = {|\(\(on|} ^ "[ \t]+" ^ {|\)?\(|} ^ numeric_slash_date ^ {|\||} ^ iso_date ^ {|\)\)|}
 let short_date =
-   "\\(\\(today\\)\\|\\(tomorrow\\)\\|" ^
-   "\\(in[ \t]+\\([0-9]+\\)[ \t]+day\\(s\\|s' time\\)?\\)\\)"
+   {|\(\(today\)\|\(tomorrow\)\||} ^
+   {|\(in|} ^ "[ \t]+" ^ {|\([0-9]+\)|} ^ "[ \t]+" ^ {|day\(s\|s' time\)?\)\)|}
 let date_spec =
-   "\\(" ^ weekdays ^ "\\|" ^ numeric_date ^ "\\|" ^ short_date ^ "\\)"
+   {|\(|} ^ weekdays ^ {|\||} ^ numeric_date ^ {|\||} ^ short_date ^ {|\)|}
 
 (* 5, or 5:30, or 5pm, or 5:30 pm *)
 let time_spec =
-   "\\(\\(\\([0-9][0-9]?\\)\\(:\\([0-9][0-9]\\)\\)?[ \t]*\\(am\\|pm\\)?\\)\\|" ^
-   "\\(noon\\|midnight\\)\\)"
+   {|\(\(\([0-9][0-9]?\)\(:\([0-9][0-9]\)\)?|} ^ "[ \t]*" ^ {|\(am\|pm\)?\)\||} ^
+   {|\(noon\|midnight\)\)|}
 (* either match a single time spec, or match a pair formatted as a range:
  * 5-7 or 5:30pm-7 or 5 until 7 or 5:30 to 7pm ... *)
 let time_range_spec =
-   "\\(\\(at[ \t]+\\)?" ^ time_spec ^ "\\([ \t]*\\(-\\|to\\|\\until\\)[ \t]*" ^
-   time_spec ^ "\\)?\\)"
+   {|\(\(at|} ^ "[ \t]+" ^ {|\)?|} ^ time_spec ^ {|\(|} ^ "[ \t]*" ^ {|\(-\|to\|\until\)|} ^ "[ \t]*" ^
+   time_spec ^ {|\)?\)|}
 
 let date_start_regex = Str.regexp_case_fold ("[ \t,.]*" ^ date_spec ^ "[ \t,.]+")
 let time_start_regex = Str.regexp_case_fold ("[ \t,.]*" ^ time_range_spec ^ "[ \t,.]+")
@@ -100,7 +99,7 @@ let next_matching_time future curr_tm hour min merid =
          start1_tm
       else
          (* if that fails, just use the same time tomorrow *)
-         let (start2, start2_tm) = get_tm 0 1 in
+         let (_start2, start2_tm) = get_tm 0 1 in
          start2_tm
    else if merid = "pm" then
       (* if "pm" is specified... *)
@@ -110,7 +109,7 @@ let next_matching_time future curr_tm hour min merid =
          start1_tm
       else
          (* if that fails, use the same time tomorrow *)
-         let (start2, start2_tm) = get_tm 12 1 in
+         let (_start2, start2_tm) = get_tm 12 1 in
          start2_tm
    else
       (* if neither "am" nor "pm" are specified... *)
@@ -125,7 +124,7 @@ let next_matching_time future curr_tm hour min merid =
             start2_tm
          else
             (* if that fails, shift to tomorrow morning *)
-            let (start3, start3_tm) = get_tm 0 1 in
+            let (_start3, start3_tm) = get_tm 0 1 in
             start3_tm
 
 
@@ -264,7 +263,7 @@ let parse_natural_language_date date_str =
             let _ = get_field 12 in
             find_next_wday current_tm 6 has_next
          with Not_found ->
-            parse_failwith "please submit a bug report for \"unreachable case 3\"."
+            parse_failwith {|please submit a bug report for "unreachable case 3".|}
       in
 
       let handle_numeric_slash () =
@@ -302,7 +301,7 @@ let parse_natural_language_date date_str =
             else
                parse_fail_default ()
          with Not_found ->
-            parse_failwith "please submit a bug report for \"unreachable case 4\"."
+            parse_failwith {|please submit a bug report for "unreachable case 4".|}
       in
 
       let handle_iso () =
@@ -327,7 +326,7 @@ let parse_natural_language_date date_str =
             else
                parse_fail_default ()
          with Not_found ->
-            parse_failwith "please submit a bug report for \"unreachable case 5\"."
+            parse_failwith {|please submit a bug report for "unreachable case 5".|}
       in
 
       let handle_short () =
@@ -366,7 +365,7 @@ let parse_natural_language_date date_str =
             let (_, norm) = Unix.mktime temp in
             norm
          with Not_found ->
-            parse_failwith "please submit a bug report for \"unreachable case 6\"."
+            parse_failwith {|please submit a bug report for "unreachable case 6".|}
       in
 
       let date_tm =
@@ -390,11 +389,11 @@ let parse_natural_language_date date_str =
             let _ = get_field 25 in
             handle_short ()
          with Not_found ->
-            parse_failwith "please submit a bug report for \"unreachable case 2\"."
+            parse_failwith {|please submit a bug report for "unreachable case 2".|}
       in
       date_tm
    end else
-      parse_failwith "please submit a bug report for \"unreachable case 1\"."
+      parse_failwith {|please submit a bug report for "unreachable case 1".|}
 
 
 
@@ -500,7 +499,7 @@ let parse_natural_language_time future current_tm time_str =
 
 
    end else
-      parse_failwith "please submit a bug report for \"unreachable case 10\"."
+      parse_failwith {|please submit a bug report for "unreachable case 10".|}
 
 
 (* at the beginning of the string, searches for date, then time,
